﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;

namespace WpfBindingCommon
{
    public sealed class MefCompositionContainer
    {
        private AggregateCatalog AggregateCatalog { get; set; }

        private CompositionContainer CompositionContainer { get; set; }

        public static MefCompositionContainer Instance { get; private set; }

        static MefCompositionContainer()
        {
            Instance = new MefCompositionContainer();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MefCompositionContainer"/> class.
        /// </summary>
        private MefCompositionContainer()
        {
            AggregateCatalog = new AggregateCatalog();
            CompositionContainer = new CompositionContainer(AggregateCatalog);
        }

        public void AddAssembly(string assemblyFileName)
        {
            if (string.IsNullOrWhiteSpace(assemblyFileName))
                throw new ArgumentNullException("assemblyFileName");

            AggregateCatalog.Catalogs.Add(new AssemblyCatalog(assemblyFileName));
        }

        public void Compose(params object[] targets)
        {
            if (AggregateCatalog.Catalogs.Count == 0)
                return;

            CompositionContainer.ComposeParts(targets);
        }
    }
}
